import 'dart:io';

import 'package:image/image.dart';
import 'package:launcher_icons_generator/file_writters/file_writter.dart';

import '../../utils/image_decoder.dart';
import 'android_icon_template.dart';
import 'android_paths.dart' as android_paths;
import 'xml_templates.dart' as xml_templates;
import 'package:path/path.dart' as path;

class AndroidFileWritter extends FileWritter {
  static const iconTemplates = [
    IconTemplate(sizeDirectory: 'mipmap-mdpi', size: 48),
    IconTemplate(sizeDirectory: 'mipmap-hdpi', size: 72),
    IconTemplate(sizeDirectory: 'mipmap-xhdpi', size: 96),
    IconTemplate(sizeDirectory: 'mipmap-xxhdpi', size: 144),
    IconTemplate(sizeDirectory: 'mipmap-xxxhdpi', size: 192),
  ];

  static const foregroundIconTemplates = [
    ForegroundIconTemplate(sizeDirectory: 'drawable-mdpi', size: 108),
    ForegroundIconTemplate(sizeDirectory: 'drawable-hdpi', size: 162),
    ForegroundIconTemplate(sizeDirectory: 'drawable-xhdpi', size: 216),
    ForegroundIconTemplate(sizeDirectory: 'drawable-xxhdpi', size: 324),
    ForegroundIconTemplate(sizeDirectory: 'drawable-xxxhdpi', size: 432),
  ];

  static final backgroundIconTemplates = foregroundIconTemplates
      .map((template) => BackgroundIconTemplate(
          size: template.size, sizeDirectory: template.sizeDirectory))
      .toList();

  final _decoder = ImageDecoder();

  Future<void> generate({
    required File iconFile,
    required File? foregroundFile,
    required File? backgroundFile,
  }) async {
    _generateIcon(iconFile);
    _maybeGenerateAdaptiveIcons(foregroundFile, backgroundFile);
  }

  Future<void> _generateIcon(File iconFile) async {
    if (!iconFile.existsSync()) {
      throw 'icon file ${iconFile.path} not found';
    }
    final icon = await _decoder.decodeImageFile(iconFile);
    await _writeImageFromTemplates(icon, iconTemplates);
  }

  Future<void> _writeImageFromTemplates(
    Image image,
    List<IconTemplate> templates,
  ) async {
    final allProgress = <Future>[];
    for (final template in templates) {
      final size = template.size;
      final path = template.buildPath();
      final resizedImage = _decoder.createResizedImage(size, image);
      final progress = writeIconFile(path, resizedImage);
      allProgress.add(progress);
    }
    await Future.wait<void>(allProgress);
  }

  Future<void> _maybeGenerateAdaptiveIcons(
    File? foregroundFile,
    File? backgroundFile,
  ) async {
    final isForegroundPresent =
        foregroundFile != null && foregroundFile.existsSync();
    final isBackgroundPresent =
        backgroundFile != null && backgroundFile.existsSync();
    if (isForegroundPresent && isBackgroundPresent) {
      final foreground = await _decoder.decodeImageFile(foregroundFile);
      final background = await _decoder.decodeImageFile(backgroundFile);
      await _writeImageFromTemplates(foreground, foregroundIconTemplates);
      await _writeImageFromTemplates(background, backgroundIconTemplates);
      await _writeAdaptiveIconXmlFile();
    }
  }

  Future<void> _writeAdaptiveIconXmlFile() async {
    final path = android_paths.findAdaptiveXmlPath();
    final file = await File(path).create(recursive: true);
    await file.writeAsString(xml_templates.adaptiveIconXml);
  }
}
