import 'dart:io';

import 'package:launcher_icons_generator/file_writters/android/android_file_writter.dart';
import 'package:launcher_icons_generator/file_writters/web/web_file_writter.dart';
import 'package:path/path.dart' as path;

import 'assets_extractors/android/android_asset_extractor.dart';
import 'assets_extractors/ios/ios_asset_extractor.dart';
import 'assets_extractors/web/web_asset_extractor.dart';
import 'file_writters/ios/ios_file_writer.dart';

// Note to reader to understand the architecture:

// The `LauncherIconGenerator` use two types of classes for each platform:
// an AssetExtractor which retrieves the files under the assets directory
// and a FileWritter, which converts the files retrieved by the asset extractor
// to images of the correct size and places them in the correct platform directory

// For each platform `LauncherIconGenerator` gets the assets via the platform
// Asset extractor, if there is no assets, it uses the default icon to generate
// the icons for that platform via the platform FileWritter.

class LauncherIconGenerator {
  static final _launcherIconsDirPath = path.join('assets', 'launcher_icons');
  static final _defaultIconPath =
      path.join(_launcherIconsDirPath, 'ic_launcher.png');

  final AndroidAssetsExtractor _androidAssetsExtractor =
      AndroidAssetsExtractor(_launcherIconsDirPath);
  final AndroidFileWritter _androidFileWritter = AndroidFileWritter();

  final WebAssetExtractor _webAssetExtractor =
      WebAssetExtractor(_launcherIconsDirPath);
  final WebFileWritter _webFileWritter = WebFileWritter();

  final IosAssetExtractor _iosAssetExtractor =
      IosAssetExtractor(_launcherIconsDirPath);
  final IosFileWritter _iosFileWritter = IosFileWritter();

  Future<void> generateIcons() async {
    final defaultIcon = File(_defaultIconPath);
    await generateAndroidIcons(defaultIcon);
    await generateWebIcons(defaultIcon);
    await generateIosIcons(defaultIcon);
  }

  Future<void> generateAndroidIcons(File defaultIcon) async {
    final assets = await _androidAssetsExtractor.findAll();
    await _androidFileWritter.generate(
      iconFile: assets?.icon ?? defaultIcon,
      foregroundFile: assets?.foreground,
      backgroundFile: assets?.background,
    );
  }

  Future<void> generateWebIcons(File defaultIcon) async {
    final assets = await _webAssetExtractor.findAll();
    await _webFileWritter.generate(icon: assets?.icon ?? defaultIcon);
  }

  Future<void> generateIosIcons(File defaultIcon) async {
    final assets = await _iosAssetExtractor.findAll();
    await _iosFileWritter.generate(iconFile: assets?.icon ?? defaultIcon);
  }
}
